/// <reference path="../typings/tsd.d.ts" />
/// <reference path="../node_modules/typescript/lib/typescript.d.ts" />
/// <reference path="../node_modules/immutable/dist/immutable.d.ts"/>
var ts = require("typescript");
var path = require("path");
var _ = require('lodash');
var fs_1 = require('fs');
var formatter_1 = require('./formatter');
var ejs = require('ejs');
function walker(sourceFile) {
    var interfaces = [];
    var walk = function (node) {
        switch (node.kind) {
            case 212:
                interfaces.push(node);
        }
        ts.forEachChild(node, walk);
    };
    walk(sourceFile);
    return interfaces;
}
exports.walker = walker;
function generate(fileName, text, extOptions) {
    var sourceFile = ts.createSourceFile(fileName, text.toString(), 2, false);
    var templatesFolder = path.join(__dirname, '../templates');
    var ifaces = walker(sourceFile);
    var template = fs_1.readFileSync(path.join(templatesFolder, 'file.ejs')).toString();
    var importName = path.basename(fileName.replace(/.tsx?$/, ''));
    var options = {
        ifaces: ifaces,
        importName: importName,
        indexerType: 'any',
        emitRecords: false,
        emitMarkers: false,
        emitEmptyRecords: false,
        defaultEmptyType: 'null',
        exportDeps: []
    };
    _.assign(options, extOptions);
    var functions = {
        type: function (member) {
            return sourceFile.text.slice(member.type.pos + 1, member.type.end);
        },
        isLocalType: function (typeName) {
            return _.any(ifaces, function (iface) { return iface.name.text == typeName; });
        },
        isArrayType: function (typeName) {
            return typeName.indexOf('[]') !== -1;
        },
        sanitizeTypeName: function (typeName) {
            return typeName.replace(/\[\]$/, '');
        },
        typeMap: function (member) {
            var typeName = functions.type(member);
            var sanitizedTypeName = functions.sanitizeTypeName(typeName);
            var isArrayType = functions.isArrayType(typeName);
            var isLocalType = functions.isLocalType(sanitizedTypeName);
            if (isLocalType) {
                if (isArrayType) {
                    return "Immutable.List<" + sanitizedTypeName + "Map>";
                }
                else {
                    return typeName + "Map";
                }
            }
            else {
                return typeName;
            }
        },
        deps: function (ifaceName, acc) {
            if (acc === void 0) { acc = {}; }
            var iface = _.find(ifaces, function (iface) { return iface.name.text == ifaceName; });
            iface.members.forEach(function (member) {
                var memberTypeName = functions.type(member);
                var sanitizedTypeName = functions.sanitizeTypeName(memberTypeName);
                var isVisited = acc[sanitizedTypeName];
                if (!isVisited) {
                    if (functions.isLocalType(sanitizedTypeName)) {
                        acc[sanitizedTypeName] = true;
                        functions.deps(sanitizedTypeName, acc);
                    }
                }
            });
            acc[ifaceName] = true;
            return Object.keys(acc);
        },
        ownDeps: function (ifaceName, acc) {
            if (acc === void 0) { acc = {}; }
            var iface = _.find(ifaces, function (iface) { return iface.name.text == ifaceName; });
            iface.members.forEach(function (member) {
                var memberTypeName = functions.type(member);
                var sanitizedTypeName = functions.sanitizeTypeName(memberTypeName);
                if (functions.isLocalType(sanitizedTypeName)) {
                    acc[member.name.text] = memberTypeName;
                }
            });
            return acc;
        },
        initializer: function (member) {
            var typeName = functions.type(member);
            var sanitizedTypeName = functions.sanitizeTypeName(typeName);
            var isArrayType = functions.isArrayType(typeName);
            var isLocalType = functions.isLocalType(sanitizedTypeName);
            if (isLocalType) {
                if (isArrayType) {
                    return "Immutable.List<" + sanitizedTypeName + "Map>()";
                }
                else if (!member.questionToken) {
                    return "new " + typeName + "Record()";
                }
                else {
                    return options.defaultEmptyType;
                }
            }
            else {
                return options.defaultEmptyType;
            }
        },
        exportDeps: function () {
            var deps = {};
            ifaces.forEach(function (iface) {
                deps[iface.name.text] = true;
                iface.members.forEach(function (member) {
                    var memberTypeName = functions.type(member);
                    var sanitizedTypeName = functions.sanitizeTypeName(memberTypeName);
                    if (!functions.isLocalType(sanitizedTypeName)) {
                        var internalTypes = functions.extractTypes(memberTypeName);
                        console.log(memberTypeName, internalTypes);
                        internalTypes.forEach(function (type) { return deps[type] = true; });
                    }
                });
            });
            return Object.keys(deps);
        },
        extractTypes: function (typeName) {
            var types = typeName
                .match(/(?:[^a-z]|^)([A-Z]\w+)/g);
            if (types) {
                return types.map(function (type) { return type.match(/[A-Z]\w+/); }).map(function (s) { return s.toString(); });
            }
            else {
                return [];
            }
        },
        forEach: _.forEach
    };
    options.exportDeps = functions.exportDeps();
    var result = ejs.render(template, options, {
        filename: path.join(templatesFolder, 'file.ejs'),
        context: functions
    });
    return formatter_1.format(result);
}
exports.generate = generate;
