var index_1 = require('./index');
var fs_1 = require('fs');
var path = require('path');
var TsImmutablePlugin = (function () {
    function TsImmutablePlugin(options) {
        var opt = options || {};
        this.options = opt;
        this.files = opt.files ? [].concat(opt.files) : [];
        this.files = this.files.map(function (fileName) { return path.resolve(fileName); });
    }
    TsImmutablePlugin.prototype.generate = function (fileName) {
        var result = index_1.generate(fileName, fs_1.readFileSync(fileName).toString(), this.options);
        var resultFileName = fileName.replace(/\.ts$/, (this.options.suffix || '-i') + ".ts");
        if (this.options.verbose) {
            console.log("[tsimmutable]: write " + path.relative(process.cwd(), resultFileName));
        }
        fs_1.writeFileSync(resultFileName, result);
    };
    TsImmutablePlugin.prototype.generateAll = function () {
        var _this = this;
        this.files.forEach(function (fileName) {
            _this.generate(fileName);
        });
    };
    TsImmutablePlugin.prototype.apply = function (compiler) {
        var _this = this;
        compiler.plugin('run', function (compiler, callback) {
            _this.generateAll();
            callback();
        });
        compiler.plugin('watch-run', function (watching, callback) {
            var mtimes = watching.compiler.watchFileSystem.watcher.mtimes;
            if (!Object.keys(mtimes).length) {
                _this.generateAll();
            }
            else {
                _this.files.forEach(function (fileName) {
                    if (mtimes.hasOwnProperty(fileName)) {
                        _this.generate(fileName);
                    }
                });
            }
            callback();
        });
    };
    return TsImmutablePlugin;
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = TsImmutablePlugin;
module.exports = TsImmutablePlugin;
